@@ -2,7 +2,7 @@
 // multicast.cpp
 // ~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2015 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,7 +16,8 @@
 // Test that header file is self-contained.
 #include <boost/asio/ip/multicast.hpp>
 
-#include <boost/asio.hpp>
+#include <boost/asio/io_service.hpp>
+#include <boost/asio/ip/udp.hpp>
 #include "../unit_test.hpp"
 
 //------------------------------------------------------------------------------
@@ -75,7 +76,7 @@ void test()
     ip::multicast::hops hops2;
     sock.get_option(hops2);
     hops1 = 1;
-    static_cast<int>(hops1.value());
+    (void)static_cast<int>(hops1.value());
 
     // enable_loopback class.
 
@@ -84,9 +85,9 @@ void test()
     ip::multicast::enable_loopback enable_loopback2;
     sock.get_option(enable_loopback2);
     enable_loopback1 = true;
-    static_cast<bool>(enable_loopback1);
-    static_cast<bool>(!enable_loopback1);
-    static_cast<bool>(enable_loopback1.value());
+    (void)static_cast<bool>(enable_loopback1);
+    (void)static_cast<bool>(!enable_loopback1);
+    (void)static_cast<bool>(enable_loopback1.value());
   }
   catch (std::exception&)
   {
@@ -130,23 +131,37 @@ void test()
   sock_v6.bind(ep_v6, ec);
   bool have_v6 = !ec;
 
-  BOOST_CHECK(have_v4 || have_v6);
+  BOOST_ASIO_CHECK(have_v4 || have_v6);
 
-#if defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#if defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
   // Windows CE seems to have problems with some multicast group addresses.
   // The following address works on CE, but as it is not a private multicast
   // address it will not be used on other platforms.
   const ip::address multicast_address_v4 =
     ip::address::from_string("239.0.0.4", ec);
-#else // defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#else // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
   const ip::address multicast_address_v4 =
     ip::address::from_string("239.255.0.1", ec);
-#endif // defined(BOOST_WINDOWS) && defined(UNDER_CE)
-  BOOST_CHECK(!have_v4 || !ec);
+#endif // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
+  BOOST_ASIO_CHECK(!have_v4 || !ec);
 
+#if (defined(__MACH__) && defined(__APPLE__)) \
+  || defined(__FreeBSD__) \
+  || defined(__NetBSD__) \
+  || defined(__OpenBSD__)
+  const ip::address multicast_address_v6 =
+    ip::address::from_string("ff02::1%lo0", ec);
+#else // (defined(__MACH__) && defined(__APPLE__))
+      //   || defined(__FreeBSD__)
+      //   || defined(__NetBSD__)
+      //   || defined(__OpenBSD__)
   const ip::address multicast_address_v6 =
     ip::address::from_string("ff01::1", ec);
-  BOOST_CHECK(!have_v6 || !ec);
+#endif // (defined(__MACH__) && defined(__APPLE__))
+       //   || defined(__FreeBSD__)
+       //   || defined(__NetBSD__)
+       //   || defined(__OpenBSD__)
+  BOOST_ASIO_CHECK(!have_v6 || !ec);
 
   // join_group class.
 
@@ -154,30 +169,34 @@ void test()
   {
     ip::multicast::join_group join_group(multicast_address_v4);
     sock_v4.set_option(join_group, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec || ec == error::no_such_device,
+                       ec.value() << ", " << ec.message());
+
+    if (!ec)
+    {
+      // leave_group class.
+
+      ip::multicast::leave_group leave_group(multicast_address_v4);
+      sock_v4.set_option(leave_group, ec);
+      BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    }
   }
 
   if (have_v6)
   {
     ip::multicast::join_group join_group(multicast_address_v6);
     sock_v6.set_option(join_group, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-  }
+    BOOST_ASIO_CHECK_MESSAGE(!ec || ec == error::no_such_device,
+                       ec.value() << ", " << ec.message());
 
-  // leave_group class.
+    if (!ec)
+    {
+      // leave_group class.
 
-  if (have_v4)
-  {
-    ip::multicast::leave_group leave_group(multicast_address_v4);
-    sock_v4.set_option(leave_group, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-  }
-
-  if (have_v6)
-  {
-    ip::multicast::leave_group leave_group(multicast_address_v6);
-    sock_v6.set_option(leave_group, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+      ip::multicast::leave_group leave_group(multicast_address_v6);
+      sock_v6.set_option(leave_group, ec);
+      BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    }
   }
 
   // outbound_interface class.
@@ -187,7 +206,7 @@ void test()
     ip::multicast::outbound_interface outbound_interface(
         ip::address_v4::loopback());
     sock_v4.set_option(outbound_interface, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
   }
 
   if (have_v6)
@@ -198,7 +217,7 @@ void test()
     ip::multicast::outbound_interface outbound_interface(1);
 #endif
     sock_v6.set_option(outbound_interface, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
   }
 
   // hops class.
@@ -206,47 +225,47 @@ void test()
   if (have_v4)
   {
     ip::multicast::hops hops1(1);
-    BOOST_CHECK(hops1.value() == 1);
+    BOOST_ASIO_CHECK(hops1.value() == 1);
     sock_v4.set_option(hops1, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
 
     ip::multicast::hops hops2;
     sock_v4.get_option(hops2, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-    BOOST_CHECK(hops2.value() == 1);
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK(hops2.value() == 1);
 
     ip::multicast::hops hops3(0);
-    BOOST_CHECK(hops3.value() == 0);
+    BOOST_ASIO_CHECK(hops3.value() == 0);
     sock_v4.set_option(hops3, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
 
     ip::multicast::hops hops4;
     sock_v4.get_option(hops4, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-    BOOST_CHECK(hops4.value() == 0);
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK(hops4.value() == 0);
   }
 
   if (have_v6)
   {
     ip::multicast::hops hops1(1);
-    BOOST_CHECK(hops1.value() == 1);
+    BOOST_ASIO_CHECK(hops1.value() == 1);
     sock_v6.set_option(hops1, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
 
     ip::multicast::hops hops2;
     sock_v6.get_option(hops2, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-    BOOST_CHECK(hops2.value() == 1);
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK(hops2.value() == 1);
 
     ip::multicast::hops hops3(0);
-    BOOST_CHECK(hops3.value() == 0);
+    BOOST_ASIO_CHECK(hops3.value() == 0);
     sock_v6.set_option(hops3, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
 
     ip::multicast::hops hops4;
     sock_v6.get_option(hops4, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-    BOOST_CHECK(hops4.value() == 0);
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK(hops4.value() == 0);
   }
 
   // enable_loopback class.
@@ -254,85 +273,85 @@ void test()
   if (have_v4)
   {
     ip::multicast::enable_loopback enable_loopback1(true);
-    BOOST_CHECK(enable_loopback1.value());
-    BOOST_CHECK(static_cast<bool>(enable_loopback1));
-    BOOST_CHECK(!!enable_loopback1);
+    BOOST_ASIO_CHECK(enable_loopback1.value());
+    BOOST_ASIO_CHECK(static_cast<bool>(enable_loopback1));
+    BOOST_ASIO_CHECK(!!enable_loopback1);
     sock_v4.set_option(enable_loopback1, ec);
-#if defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#if defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
     // Option is not supported under Windows CE.
-    BOOST_CHECK_MESSAGE(ec == boost::asio::error::no_protocol_option,
+    BOOST_ASIO_CHECK_MESSAGE(ec == boost::asio::error::no_protocol_option,
         ec.value() << ", " << ec.message());
-#else // defined(BOOST_WINDOWS) && defined(UNDER_CE)
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-#endif // defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#else // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+#endif // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
 
     ip::multicast::enable_loopback enable_loopback2;
     sock_v4.get_option(enable_loopback2, ec);
-#if defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#if defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
     // Not supported under Windows CE but can get value.
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-#else // defined(BOOST_WINDOWS) && defined(UNDER_CE)
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-    BOOST_CHECK(enable_loopback2.value());
-    BOOST_CHECK(static_cast<bool>(enable_loopback2));
-    BOOST_CHECK(!!enable_loopback2);
-#endif // defined(BOOST_WINDOWS) && defined(UNDER_CE)
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+#else // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK(enable_loopback2.value());
+    BOOST_ASIO_CHECK(static_cast<bool>(enable_loopback2));
+    BOOST_ASIO_CHECK(!!enable_loopback2);
+#endif // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
 
     ip::multicast::enable_loopback enable_loopback3(false);
-    BOOST_CHECK(!enable_loopback3.value());
-    BOOST_CHECK(!static_cast<bool>(enable_loopback3));
-    BOOST_CHECK(!enable_loopback3);
+    BOOST_ASIO_CHECK(!enable_loopback3.value());
+    BOOST_ASIO_CHECK(!static_cast<bool>(enable_loopback3));
+    BOOST_ASIO_CHECK(!enable_loopback3);
     sock_v4.set_option(enable_loopback3, ec);
-#if defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#if defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
     // Option is not supported under Windows CE.
-    BOOST_CHECK_MESSAGE(ec == boost::asio::error::no_protocol_option,
+    BOOST_ASIO_CHECK_MESSAGE(ec == boost::asio::error::no_protocol_option,
         ec.value() << ", " << ec.message());
-#else // defined(BOOST_WINDOWS) && defined(UNDER_CE)
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-#endif // defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#else // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+#endif // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
 
     ip::multicast::enable_loopback enable_loopback4;
     sock_v4.get_option(enable_loopback4, ec);
-#if defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#if defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
     // Not supported under Windows CE but can get value.
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-#else // defined(BOOST_WINDOWS) && defined(UNDER_CE)
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-    BOOST_CHECK(!enable_loopback4.value());
-    BOOST_CHECK(!static_cast<bool>(enable_loopback4));
-    BOOST_CHECK(!enable_loopback4);
-#endif // defined(BOOST_WINDOWS) && defined(UNDER_CE)
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+#else // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK(!enable_loopback4.value());
+    BOOST_ASIO_CHECK(!static_cast<bool>(enable_loopback4));
+    BOOST_ASIO_CHECK(!enable_loopback4);
+#endif // defined(BOOST_ASIO_WINDOWS) && defined(UNDER_CE)
   }
 
   if (have_v6)
   {
     ip::multicast::enable_loopback enable_loopback1(true);
-    BOOST_CHECK(enable_loopback1.value());
-    BOOST_CHECK(static_cast<bool>(enable_loopback1));
-    BOOST_CHECK(!!enable_loopback1);
+    BOOST_ASIO_CHECK(enable_loopback1.value());
+    BOOST_ASIO_CHECK(static_cast<bool>(enable_loopback1));
+    BOOST_ASIO_CHECK(!!enable_loopback1);
     sock_v6.set_option(enable_loopback1, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
 
     ip::multicast::enable_loopback enable_loopback2;
     sock_v6.get_option(enable_loopback2, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-    BOOST_CHECK(enable_loopback2.value());
-    BOOST_CHECK(static_cast<bool>(enable_loopback2));
-    BOOST_CHECK(!!enable_loopback2);
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK(enable_loopback2.value());
+    BOOST_ASIO_CHECK(static_cast<bool>(enable_loopback2));
+    BOOST_ASIO_CHECK(!!enable_loopback2);
 
     ip::multicast::enable_loopback enable_loopback3(false);
-    BOOST_CHECK(!enable_loopback3.value());
-    BOOST_CHECK(!static_cast<bool>(enable_loopback3));
-    BOOST_CHECK(!enable_loopback3);
+    BOOST_ASIO_CHECK(!enable_loopback3.value());
+    BOOST_ASIO_CHECK(!static_cast<bool>(enable_loopback3));
+    BOOST_ASIO_CHECK(!enable_loopback3);
     sock_v6.set_option(enable_loopback3, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
 
     ip::multicast::enable_loopback enable_loopback4;
     sock_v6.get_option(enable_loopback4, ec);
-    BOOST_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
-    BOOST_CHECK(!enable_loopback4.value());
-    BOOST_CHECK(!static_cast<bool>(enable_loopback4));
-    BOOST_CHECK(!enable_loopback4);
+    BOOST_ASIO_CHECK_MESSAGE(!ec, ec.value() << ", " << ec.message());
+    BOOST_ASIO_CHECK(!enable_loopback4.value());
+    BOOST_ASIO_CHECK(!static_cast<bool>(enable_loopback4));
+    BOOST_ASIO_CHECK(!enable_loopback4);
   }
 }
 
@@ -340,10 +359,9 @@ void test()
 
 //------------------------------------------------------------------------------
 
-test_suite* init_unit_test_suite(int, char*[])
-{
-  test_suite* test = BOOST_TEST_SUITE("ip/multicast");
-  test->add(BOOST_TEST_CASE(&ip_multicast_compile::test));
-  test->add(BOOST_TEST_CASE(&ip_multicast_runtime::test));
-  return test;
-}
+BOOST_ASIO_TEST_SUITE
+(
+  "ip/multicast",
+  BOOST_ASIO_TEST_CASE(ip_multicast_compile::test)
+  BOOST_ASIO_TEST_CASE(ip_multicast_runtime::test)
+)
